{-----------------------------------------------------------------------------
 Unit Name: CocoGen3
 Author:    Patrick Connors
 Purpose:
 Class that controls the COCO/R Parser for ParserBuilder.

 Modelled on the main unit of COCO/R for TurboPascal 1.51 by
 Prof Pat Terry and Volker Pohlers.

 Planning to open up the internal COCO/R constants as properties
 that can be altered to tune the generated parsers.
 - in the meantime, there are commented-out properties in this class for now.
-----------------------------------------------------------------------------}
unit CocoGen3;

interface

uses
  Windows, Messages, SysUtils, Classes, Dialogs, Forms,
  pbCocoScanner, pbStrLib, pbCocoParser, pbCocoTable,
  pbCocoSets, pbCocoAutomaton, pbCocoParserGen, pbCocoDriverGen,
  pbCocoConstants, JclFileUtils;

type
  CHARSET = set of char;
  Err = ^ErrDesc;
  ErrDesc = record
    nr, line, col: Integer;
    Next: Err
  end;

type
  TCocoGen3 = class(TObject)
  private
    { Private declarations }
    firstErr, lastErr: Err;
    Extra: Integer;
    FGrammarName: string;
    FlstFileName: string;
    FOptions: string;
    FATGFileName: string;
    fpbCocoParser: TpbCocoParser;
    fpbCocoTable: TpbCocoTable;
    fpbCocoAutomaton: TpbCocoAutomaton;
    fpbCocoParserGen: TpbCocoParserGen;
    fpbCocoDriverGen: TpbCocoDriverGen;
    ll1: boolean;
    FFramesDirectory: string;
    FOutputDirectory: string;
    FSucceeded: boolean; (* TRUE, if grammar is LL(1) *)
    FErrorFormat: string;
    //    FmaxAlter: Integer;
    //    FmaxNodes: Integer;
    //    FsymSetSize: Integer;
    //    FmaxLiterals: Integer;
    //    FmaxNt: Integer;
    //    FmaxSemLen: Integer;
    //    FmaxSymbols: Integer;
    //    FmaxTerm: Integer;
    //    FMaxNameLength: Integer;
    //    FmaxTerminals: Integer;
    //    FMaxSourceLineLength: Integer;
    //    FcontextTrans: Integer;
    //    FnormTrans: Integer;
    //    FmaxList: Integer;
    //    FmaxClasses: Integer;
    //    FSetSize: Integer;
    //    FSetLimit: Integer;
    FPBVersion: string;
    procedure StoreError(nr, line, col: Integer; pos: longint);
    procedure GetLine(var ipos: longint; var line: string; var EOF: boolean);
    procedure PrintErr(line: string; nr, col: Integer);
    procedure PrintListing;
    procedure SetOption(s: string);
    procedure DoExecute;
    procedure SetATGFileName(const Value: string);
    procedure SetGrammarName(const Value: string);
    procedure SetlstFileName(const Value: string);
    procedure SetOptions(const Value: string);
    procedure Msgout(const S: string);
    procedure SetFramesDirectory(const Value: string);
    procedure SetOutputDirectory(const Value: string);
    function GetOutputStream: TStringStream;
    function GetSrcStream: TStringStream;
    function GetCompilerStream: TStringStream;
    function GetParserStream: TStringStream;
    function GetScannerStream: TStringStream;
    function GetErrorStream: TStringStream;
    procedure SetSucceeded(const Value: boolean);
    procedure SetErrorFormat(const Value: string);
    //    procedure SetcontextTrans(const Value: Integer);
    //    procedure SetmaxAlter(const Value: Integer);
    //    procedure SetmaxClasses(const Value: Integer);
    //    procedure SetmaxList(const Value: Integer);
    //    procedure SetmaxLiterals(const Value: Integer);
    //    procedure SetMaxNameLength(const Value: Integer);
    //    procedure SetmaxNodes(const Value: Integer);
    //    procedure SetmaxNt(const Value: Integer);
    //    procedure SetmaxSemLen(const Value: Integer);
    //    procedure SetMaxSourceLineLength(const Value: Integer);
    //    procedure SetmaxSymbols(const Value: Integer);
    //    procedure SetmaxTerm(const Value: Integer);
    //    procedure SetmaxTerminals(const Value: Integer);
    //    procedure SetnormTrans(const Value: Integer);
    //    procedure SetsymSetSize(const Value: Integer);
    //    procedure SetDefaultConstants;
    procedure CreateSubObjects;
    procedure DestroySubObjects;
    //    procedure SetSetLimit(const Value: Integer);
    //    procedure SetSetSize(const Value: Integer);
    function GetVersion: string;
    function getPBVersion: string;
    procedure setPBVersion(const Value: string);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create;
    destructor Destroy; override;
    procedure Execute;
  published
    { Published declarations }
    property Options: string read FOptions write SetOptions;
    property GrammarName: string read FGrammarName write SetGrammarName;
    property ATGFileName: string read FATGFileName write SetATGFileName;
    property lstFileName: string read FlstFileName write SetlstFileName;
    property FramesDirectory: string read FFramesDirectory write SetFramesDirectory;
    property OutputDirectory: string read FOutputDirectory write SetOutputDirectory;
    property OutputStream: TStringStream read GetOutputStream;
    property SrcStream: TStringStream read GetSrcStream;
    property ParserStream: TStringStream read GetParserStream;
    property ScannerStream: TStringStream read GetScannerStream;
    property ErrorStream: TStringStream read GetErrorStream;
    property CompilerStream: TStringStream read GetCompilerStream;
    property Succeeded: boolean read FSucceeded write SetSucceeded;
    property ErrorFormat: string read FErrorFormat write SetErrorFormat;

    // The following Properties will be opened up when I implement
    // the fundamental constants as properties.
    // This will allow further tuning of the generated code.

    //    (* max number of symbols (terminals+nonterminals+pragmas) *)
    //    property maxSymbols: Integer read FmaxSymbols write SetmaxSymbols;
    //    (* max number of terminals *)
    //    property maxTerminals: Integer read FmaxTerminals write SetmaxTerminals;
    //    (* max number of nonterminals *)
    //    property maxNt: Integer read FmaxNt write SetmaxNt;
    //    (* max number of top-down graph nodes *)
    //    property maxNodes: Integer read FmaxNodes write SetmaxNodes;
    //    (* max number of character classes *)
    //    property maxClasses: Integer read FmaxClasses write SetmaxClasses;
    //    (* max length of a semantic text *)
    //    property maxSemLen: Integer read FmaxSemLen write SetmaxSemLen;
    //    (* DFA transition during normal scanning *)
    //    property normTrans: Integer read FnormTrans write SetnormTrans;
    //    (* DFA transition during scanning of right context *)
    //    property contextTrans: Integer read FcontextTrans write SetcontextTrans;
    //    (* max array size in FindCircularProductions *)
    //    property maxList: Integer read FmaxList write SetmaxList;
    //    (* max number of literal terminals *)
    //    property maxLiterals: Integer read FmaxLiterals write SetmaxLiterals;
    //    (* max length of token name *)
    //    property MaxNameLength: Integer read FMaxNameLength write SetMaxNameLength;
    //    (* max.number of symbol sets of the generated parser *)
    //    property symSetSize: Integer read FsymSetSize write SetsymSetSize;
    //    (* sets of size < maxTerm are enumerated *)
    //    property maxTerm: Integer read FmaxTerm write SetmaxTerm;
    //    (* more than maxAlter alternatives are handled with  a case statement *)(* kinds of generated error messages *)
    //    property maxAlter: Integer read FmaxAlter write SetmaxAlter;
    //    property MaxSourceLineLength: Integer read FMaxSourceLineLength         
    //      write SetMaxSourceLineLength;
    //    property SetSize: Integer read FSetSize write SetSetSize;
    //    property SetLimit: Integer read FSetLimit write SetSetLimit;
    property PBVersion: string read getPBVersion write setPBVersion;
  end;

implementation

{ TCocoGen3 }

constructor TCocoGen3.Create;
begin
  inherited;
  //  SetDefaultConstants;
end;

procedure TCocoGen3.CreateSubObjects;
begin
  DestroySubObjects;
  fpbCocoParser := TpbCocoParser.Create;
  fpbCocoTable := TpbCocoTable.Create;
  fpbCocoAutomaton := TpbCocoAutomaton.Create;
  fpbCocoParserGen := TpbCocoParserGen.Create;
  fpbCocoDriverGen := TpbCocoDriverGen.Create;
  fpbCocoParser.CocoTable := fpbCocoTable;
  fpbCocoParser.CocoAutomaton := fpbCocoAutomaton;
  fpbCocoTable.CocoScanner := fpbCocoParser;
  fpbCocoAutomaton.CocoScanner := fpbCocoParser;
  fpbCocoAutomaton.CocoTable := fpbCocoTable;
  fpbCocoParserGen.CocoScanner := fpbCocoParser;
  fpbCocoParserGen.CocoTable := fpbCocoTable;
  fpbCocoParserGen.CocoAutomaton := fpbCocoAutomaton;
  fpbCocoDriverGen.CocoScanner := fpbCocoParser;
  fpbCocoDriverGen.CocoTable := fpbCocoTable;
  fpbCocoDriverGen.CocoAutomaton := fpbCocoAutomaton;
  fpbCocoParserGen.ErrorFormat := fErrorFormat;
  fpbCocoParser.Error := StoreError;
  fpbCocoParser.directory := fFramesDirectory;
  SetOption(FOptions);
end;

destructor TCocoGen3.Destroy;
begin
  DestroySubObjects;
  inherited;
end;

procedure TCocoGen3.DestroySubObjects;
begin
  fpbCocoParser.Free;
  fpbCocoTable.Free;
  fpbCocoAutomaton.Free;
  fpbCocoParserGen.Free;
  fpbCocoDriverGen.Free;

  fpbCocoParser := nil;
  fpbCocoTable := nil;
  fpbCocoAutomaton := nil;
  fpbCocoParserGen := nil;
  fpbCocoDriverGen := nil;
end;

procedure TCocoGen3.DoExecute;
var
  OK: boolean;
  fn: string;
  S, ClassName: string;
begin
  fFramesDirectory := ExtractFilePath(Application.ExeName) + '\Frames';
  if not DirectoryExists(fOutputDirectory) then
    fOutputDirectory := ExtractFilePath(Application.ExeName) + '\Output';
  CreateSubObjects;
  fpbCocoParser.OutputDirectory := FOutputDirectory;
  firstErr := nil;
  Extra := 1;
  if not (FileExists(GrammarName)) then
    raise Exception.Create('Grammar filename not set');
  LoadString(S, GrammarName);
  ClassName := ExtractFileName(GrammarName);
  ClassName := ChangeFileExt(ClassName, '');
  S := ReplaceAll(S, '-->Grammar', ClassName);
  fpbCocoParser.SourceString := S;
  { TODO : Work out appropriate header for the report output. }
  pbWriteln(fpbCocoParser.lst, HeaderLine);
  pbWrite(fpbCocoParser.lst, PBVersion);
  pbWriteln(fpbCocoParser.lst, '');
  pbWriteLn(fpbCocoParser.lst, 'Parsing ' + FGrammarName);
  pbWriteln(fpbCocoParser.lst, '');
  fpbCocoParser.Error := StoreError;
  fpbCocoParser.Parse;
  if fpbCocoParser.errors = 0 then
  begin
    FSucceeded := True;
    MsgOut('Syntax OK, now testing grammar ......');
    pbWriteln(fpbCocoParser.lst, '');
    pbWriteLn(fpbCocoParser.lst, '');
    fpbCocoTable.CompSymbolSets;
    pbWriteLn(fpbCocoParser.lst, '');
    fpbCocoTable.TestCompleteness(ok);
    if ok then fpbCocoTable.TestIfAllNtReached(ok);
    if ok then fpbCocoTable.FindCircularProductions(ok);
    if ok then fpbCocoTable.TestIfNtToTerm(ok);
    MsgOut('');
    if not OK then MsgOut('Grammar tests failed.')
    else
      MsgOut('Grammar tests OK.');
    MsgOut('');
    if ok then fpbCocoTable.LL1Test(ll1);
    pbWriteLn(fpbCocoParser.lst, '');
    FSucceeded := (ok and ll1);

    if not ok or not ll1 or fpbCocoTable.ddt['L'] or fpbCocoTable.ddt['X'] then
    begin
      MsgOut('');

      MsgOut('----------------------------------------------');
      PrintListing;
      //pbWriteLn(fpbCocoParser.lst, '');
      if fpbCocoTable.ddt['X'] then fpbCocoTable.XRef;
      pbWriteLn(fpbCocoParser.lst, '');
    end;
    if fpbCocoTable.ddt['N'] or fpbCocoTable.symNames then
    begin
      MsgOut('Symbol Name Assignments:');
      fpbCocoTable.AssignSymNames(fpbCocoTable.ddt['N'], fpbCocoTable.FsymNames);
      pbWriteLn(fpbCocoParser.lst, '');
    end;
    if ok and not fpbCocoTable.ddt['T'] then
    begin
      MsgOut('Generating Parser');
      fpbCocoParserGen.GenCompiler;
      if fpbCocoTable.genScanner and not fpbCocoTable.ddt['P'] then
      begin
        MsgOut('Generating Scanner');
        fpbCocoAutomaton.FramInPos := 0;
        fpbCocoAutomaton.WriteScanner(ok);
        pbWriteLn(fpbCocoParser.lst, '');
        if fpbCocoTable.ddt['A'] then fpbCocoAutomaton.PrintStates;
      end;
      if fpbCocoTable.ddt['C'] then
      begin
        MsgOut('Generating Compiler');
        fpbCocoDriverGen.WriteDriver;
        pbWriteLn(fpbCocoParser.lst, '');
      end;
      fpbCocoParserGen.WriteStatistics;
    end;
    if not ok then MsgOut('Compilation ended with errors in grammar tests.')
    else if not ll1 then MsgOut('Compilation ended with LL(1) errors.')
    else
      MsgOut('Compilation completed. No errors detected.');
  end
  else
  begin
    MsgOut('//---------- Parse Failed ------------------');
    PrintListing;
    FSucceeded := False;
    if fpbCocoTable.ddt['X'] then fpbCocoTable.XRef;
    MsgOut('*** errors detected ***');
  end;
  if fpbCocoTable.ddt['G'] then fpbCocoTable.PrintGraph;
  if fpbCocoTable.ddt['S'] then fpbCocoTable.PrintSymbolTable;
  pbWriteln(fpbCocoParser.lst, HeaderLine);
  fn := ExtractFileName(GrammarName);
  fn := fpbCocoParser.OutputDirectory + '\' + fn;
  fn := ChangeFileExt(fn, '.lst');
  SaveStringStream(fpbCocoParser.lst, fn);
end;

procedure TCocoGen3.Execute;
begin
  DoExecute;
end;

function TCocoGen3.GetCompilerStream: TStringStream;
begin
  Result := fpbCocoDriverGen.com;
end;

function TCocoGen3.GetErrorStream: TStringStream;
begin
  Result := fpbCocoParserGen.err;
end;


procedure TCocoGen3.GetLine(var ipos: Integer; var line: string;
  var EOF: boolean);
var
  ch: char;
  i: integer;
begin
  i := 1;
  EOF := False;
  setLength(Line, 1024); //was 255 - pc
  fpbCocoParser.CharAt(ipos, ch);
  INC(ipos);
  while (i < 1024) and not (ch in [#13, #10, #0]) do
  begin
    line[i] := ch;
    INC(i);
    fpbCocoParser.CharAt(ipos, ch);
    INC(ipos);
  end;
  setLength(Line, i - 1);
  EOF := (i = 1) and (ch = _EF);
  if ch = _CR then
  begin
    fpbCocoParser.CharAt(ipos, ch);
    if ch = _LF then
    begin
      INC(ipos);
      Extra := 0
    end
  end;
end;

function TCocoGen3.GetOutputStream: TStringStream;
begin
  Result := fpbCocoParser.lst;
end;

function TCocoGen3.GetParserStream: TStringStream;
begin
  Result := fpbCocoParserGen.syn;
end;

function TCocoGen3.getPBVersion: string;
begin
  if fPBVersion = '' then
    FPBVersion := GetVersion;
  Result := fPBVersion;
end;

function TCocoGen3.GetScannerStream: TStringStream;
begin
  Result := fpbCocoAutomaton.GenScannerStream;
end;


function TCocoGen3.GetSrcStream: TStringStream;
begin
  Result := fpbCocoParser.src;
end;

function TCocoGen3.GetVersion: string;
var 
  VI: TJclFileVersionInfo;
begin
  VI := TJclFileVersionInfo.Create(Application.ExeName);
  Result := 'ParserBuilder for Delphi ' + VI.BinFileVersion + 'b';
  VI.Free;
end;

procedure TCocoGen3.Msgout(const S: string);
begin
  pbWriteLn(fpbCocoParser.lst, S);
end;

procedure TCocoGen3.PrintErr(line: string; nr, col: Integer);
  { TODO : Make a regular error reporting routine - this one is crappo }

  procedure MsgOut(s: string);
  begin
    pbWrite(fpbCocoParser.lst, s);
  end;

  procedure Pointer;
  var
    i: integer;
  begin
    pbWrite(fpbCocoParser.lst, '*****  ');
    i := 1;
    while i < col + Extra - 2 do
    begin
      if line[i] = TAB then
        pbWrite(fpbCocoParser.lst, TAB)
      else
        pbWrite(fpbCocoParser.lst, ' ');
      INC(i)
    end;
    pbWrite(fpbCocoParser.lst, '^ ');
  end;
begin
  Pointer;
  case nr of
    0: MsgOut('EOF expected');
    1: MsgOut('ident expected');
    2: MsgOut('string expected');
    3: MsgOut('badstring expected');
    4: MsgOut('number expected');
    5: MsgOut('"COMPILER" expected');
    6: MsgOut('"USES" expected');
    7: MsgOut('"," expected');
    8: MsgOut('";" expected');
    9: MsgOut('"DELPHICLASS" expected');
    10: MsgOut('"END_DELPHICLASS" expected');
    11: MsgOut('"PRODUCTIONS" expected');
    12: MsgOut('"=" expected');
    13: MsgOut('"." expected');
    14: MsgOut('"END" expected');
    15: MsgOut('"CHARACTERS" expected');
    16: MsgOut('"TOKENS" expected');
    17: MsgOut('"NAMES" expected');
    18: MsgOut('"PRAGMAS" expected');
    19: MsgOut('"COMMENTS" expected');
    20: MsgOut('"FROM" expected');
    21: MsgOut('"TO" expected');
    22: MsgOut('"NESTED" expected');
    23: MsgOut('"IGNORE" expected');
    24: MsgOut('"CASE" expected');
    25: MsgOut('"GLOBALDECS" expected');
    26: MsgOut('"PRIVATE" expected');
    27: MsgOut('"PROTECTED" expected');
    28: MsgOut('"PUBLIC" expected');
    29: MsgOut('"PUBLISHED" expected');
    30: MsgOut('"CREATE" expected');
    31: MsgOut('"DESTROY" expected');
    32: MsgOut('"+" expected');
    33: MsgOut('"-" expected');
    34: MsgOut('".." expected');
    35: MsgOut('"ANY" expected');
    36: MsgOut('"CHR" expected');
    37: MsgOut('"(" expected');
    38: MsgOut('")" expected');
    39: MsgOut('"|" expected');
    40: MsgOut('"WEAK" expected');
    41: MsgOut('"[" expected');
    42: MsgOut('"]" expected');
    43: MsgOut('"{" expected');
    44: MsgOut('"}" expected');
    45: MsgOut('"SYNC" expected');
    46: MsgOut('"CONTEXT" expected');
    47: MsgOut('"<" expected');
    48: MsgOut('">" expected');
    49: MsgOut('"<." expected');
    50: MsgOut('".>" expected');
    51: MsgOut('"(." expected');
    52: MsgOut('".)" expected');
    53: MsgOut('not expected');
    54: MsgOut('invalid TokenFactor');
    55: MsgOut('invalid Factor');
    56: MsgOut('invalid Factor');
    57: MsgOut('invalid Term');
    58: MsgOut('invalid Symbol');
    59: MsgOut('invalid SingleChar');
    60: MsgOut('invalid SimSet');
    61: MsgOut('invalid NameDecl');
    62: MsgOut('this symbol not expected in TokenDecl');
    63: MsgOut('invalid TokenDecl');
    64: MsgOut('invalid Attribs');
    65: MsgOut('invalid Declaration');
    66: MsgOut('invalid Declaration');
    67: MsgOut('invalid Declaration');
    68: MsgOut('invalid ClassSection');
    69: MsgOut('this symbol not expected in pbCoco');
    70: MsgOut('invalid pbCoco');
    102: MsgOut('string literal may not extend over line end');
    103: MsgOut('a literal must not have attributes');
    104: MsgOut('this symbol kind not allowed in production');
    105: MsgOut('attribute mismatch between declaration and use');
    106: MsgOut('undefined string in production');
    107: MsgOut('name declared twice');
    108: MsgOut('this type not allowed on left side of production');
    109: MsgOut('earlier semantic action was not terminated');
    111: MsgOut('missing production for grammar name');
    112: MsgOut('grammar symbol must not have attributes');
    113: MsgOut('a literal must not be declared with a structure');
    114: MsgOut('semantic action not allowed here');
    115: MsgOut('undefined name');
    116: MsgOut('attributes not allowed in token declaration');
    117: MsgOut('name does not match grammar name');
    118: MsgOut('unacceptable constant value');
    119: MsgOut('may not ignore CHR(0)');
    120: MsgOut('token may not be empty');
    121: MsgOut('token must not start with an iteration');
    122: MsgOut('only characters allowed in comment declaration');
    123: MsgOut('only terminals may be weak');
    124: MsgOut('literal tokens may not contain white space');
    125: MsgOut('comment delimiter must be 1 or 2 characters long');
    126: MsgOut('character set contains more than one character');
    127: MsgOut('could not make deterministic automaton');
    128: MsgOut('semantic action text too long - please split it');
    129: MsgOut('literal tokens may not be empty');
    130: MsgOut('IGNORE CASE must appear earlier');
    else
    begin
      MsgOut('Error: ');
      pbWrite(fpbCocoParser.lst, IntToStr(nr));
    end
  end;
  pbWriteLn(fpbCocoParser.lst, '');
  //  inc(fpbCocoParser.errors);
end;


procedure TCocoGen3.PrintListing;
  (* Print a source listing with error messages *)
var
  nextErr: Err;
  EOF: boolean;
  lnr: integer;
  srcPos: longint;
  line: string;
begin
  pbWriteLn(fpbCocoParser.lst, 'Listing:');
  pbWriteLn(fpbCocoParser.lst, '');
  srcPos := 0;
  nextErr := firstErr;
  GetLine(srcPos, line, EOF);
  lnr := 1;
  while not EOF do
  begin
    pbWriteLn(fpbCocoParser.lst, IntToStr(lnr) + #9 + line);
    while (nextErr <> nil) and (nextErr^.line = lnr) do
    begin
      PrintErr(line, nextErr^.nr, nextErr^.col);
      nextErr := nextErr^.Next
    end;
    GetLine(srcPos, line, EOF);
    INC(lnr);
  end;
  if nextErr <> nil then
  begin
    pbWriteLn(fpbCocoParser.lst, IntToStr(lnr));
    while nextErr <> nil do
    begin
      PrintErr(line, nextErr^.nr, nextErr^.col);
      nextErr := nextErr^.Next
    end
  end;
  pbWriteLn(fpbCocoParser.lst, '');
  pbWriteLn(fpbCocoParser.lst, '');
  pbWriteLn(fpbCocoParser.lst, '');
  pbWriteLn(fpbCocoParser.lst, '');
end;


procedure TCocoGen3.SetATGFileName(const Value: string);
begin
  FATGFileName := Value;
end;

//procedure TCocoGen3.SetcontextTrans(const Value: Integer);
//begin
//  FcontextTrans := Value;
//end;
//
//procedure TCocoGen3.SetDefaultConstants;
//begin
//  FmaxAlter := 5;
//  FmaxNodes := 1500;
//  FsymSetSize := 100;
//  FmaxLiterals := 127;
//  FmaxNt := 128;
//  FmaxSemLen := 64000;
//  FmaxSymbols := 256;
//  FmaxTerm := 5;
//  FMaxNameLength := 39;
//  FmaxTerminals := 256;
//  FMaxSourceLineLength := 78;
//  FcontextTrans := 1;
//  FnormTrans := 0;
//  FmaxList := 150;
//  FmaxClasses := 250;
//  fSetSize := 16;
//  fSetLimit := 32;
//end;

procedure TCocoGen3.SetErrorFormat(const Value: string);
begin
  FErrorFormat := Value;
end;


procedure TCocoGen3.SetFramesDirectory(const Value: string);
begin
  FFramesDirectory := Value;
end;

procedure TCocoGen3.SetGrammarName(const Value: string);
begin
  FGrammarName := Value;
end;

procedure TCocoGen3.SetlstFileName(const Value: string);
begin
  FlstFileName := Value;
end;

//procedure TCocoGen3.SetmaxAlter(const Value: Integer);
//begin
//  FmaxAlter := Value;
//end;
//
//procedure TCocoGen3.SetmaxClasses(const Value: Integer);
//begin
//  FmaxClasses := Value;
//end;
//
//procedure TCocoGen3.SetmaxList(const Value: Integer);
//begin
//  FmaxList := Value;
//end;
//
//procedure TCocoGen3.SetmaxLiterals(const Value: Integer);
//begin
//  FmaxLiterals := Value;
//end;
//
//procedure TCocoGen3.SetMaxNameLength(const Value: Integer);
//begin
//  FMaxNameLength := Value;
//end;
//
//procedure TCocoGen3.SetmaxNodes(const Value: Integer);
//begin
//  FmaxNodes := Value;
//end;
//
//procedure TCocoGen3.SetmaxNt(const Value: Integer);
//begin
//  FmaxNt := Value;
//end;
//
//procedure TCocoGen3.SetmaxSemLen(const Value: Integer);
//begin
//  FmaxSemLen := Value;
//end;
//
//procedure TCocoGen3.SetMaxSourceLineLength(const Value: Integer);
//begin
//  FMaxSourceLineLength := Value;
//end;
//
//procedure TCocoGen3.SetmaxSymbols(const Value: Integer);
//begin
//  FmaxSymbols := Value;
//end;
//
//procedure TCocoGen3.SetmaxTerm(const Value: Integer);
//begin
//  FmaxTerm := Value;
//end;
//
//procedure TCocoGen3.SetmaxTerminals(const Value: Integer);
//begin
//  FmaxTerminals := Value;
//end;
//
//procedure TCocoGen3.SetnormTrans(const Value: Integer);
//begin
//  FnormTrans := Value;
//end;

procedure TCocoGen3.SetOption(s: string);
var
  i: integer;
begin
  for i := 1 to 26 do
    fpbCocoTable.ddt[CHR(i + 64)] := False;

  for i := 2 to Length(s) do
  begin
    s[i] := UpCase(s[i]);
    if s[i] in ['A'..'Z'] then fpbCocoTable.ddt[s[i]] := True;
  end;
end;

procedure TCocoGen3.SetOptions(const Value: string);
begin
  FOptions := Value;
  // Will delay the actual setting of the options
  // as no longer assuming the CocoTable is created yet;
  //  SetOption(FOptions);
end;

procedure TCocoGen3.SetOutputDirectory(const Value: string);
begin
  FOutputDirectory := Value;
end;

procedure TCocoGen3.setPBVersion(const Value: string);
begin
end;

//procedure TCocoGen3.SetSetLimit(const Value: Integer);
//begin
//  FSetLimit := Value;
//end;
//
//procedure TCocoGen3.SetSetSize(const Value: Integer);
//begin
//  FSetSize := Value;
//end;

procedure TCocoGen3.SetSucceeded(const Value: boolean);
begin
  FSucceeded := Value;
end;

//procedure TCocoGen3.SetsymSetSize(const Value: Integer);
//begin
//  FsymSetSize := Value;
//end;

procedure TCocoGen3.StoreError(nr, line, col, pos: Integer);
var
  nextErr: Err;
begin
  { TODO : We store the errors but do we free them? }
  NEW(nextErr);
  nextErr^.nr := nr;
  nextErr^.line := line;
  nextErr^.col := col;
  nextErr^.Next := nil;
  if firstErr = nil then firstErr :=
      nextErr
  else
    lastErr^.Next := nextErr;
  lastErr := nextErr;
  INC(fpbCocoParser.ferrors)
end;


end.
