{-----------------------------------------------------------------------------
 Unit Name: CocoCharacters
 Author:    pat connors
 Purpose: Container for GUI-model of ATG File - Holds the Characters
-----------------------------------------------------------------------------}


unit CocoCharacters;

interface

uses
  Windows, Messages, SysUtils, Classes, pbStrLib;

type
  TCocoCharacters = class(TComponent)
  private
    FCharacterText: TStringList;
    FComments: string;
    FIsEditing: boolean;
    FEditingCharSet: string;
    procedure SetCharacterText(const Value: TStringList);
    procedure SetComments(const Value: string);
    procedure SetIsEditing(const Value: boolean);
    procedure SetEditingCharSet(const Value: string);
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetATG: string;
    procedure AddSet(const SetContents: string);
    function GetCharacterSet(const SetTitle: string): string;

  published
    { Published declarations }
    property CharacterText: TStringList read FCharacterText write SetCharacterText;
    property Comments: string read FComments write SetComments;
    property IsEditing: boolean read FIsEditing write SetIsEditing;
    property EditingCharSet: string read FEditingCharSet write SetEditingCharSet;
  end;

implementation


{ TCocoCharacters }

procedure TCocoCharacters.AddSet(const SetContents: string);
begin
  FCharacterText.Add(Trim(SetContents));
end;

constructor TCocoCharacters.Create(AOwner: TComponent);
begin
  inherited;
  FCharacterText := TStringList.Create;
end;

destructor TCocoCharacters.Destroy;
begin
  FCharacterText.Free;
  inherited;
end;

function TCocoCharacters.GetATG: string;
var
  S: string;
  idx: integer;
begin
  if not fIsEditing then
  begin
    S := Trim(FCharacterText.Text);
    if (S <> '') then
    begin
      if FEditingCharSet = '' then
      begin
        S := ReplaceAll(S, #13#10, '.' + #13#10) + '.';
        Result := 'CHARACTERS' + CRLF + S + CRLF;
        Exit;
      end
      else
      begin
        // a particular row fo the stringlist is under edit
        S := '';
        for idx := 0 to FCharacterText.Count - 1 do
        begin
          if FCharacterText.IndexOfName(FEditingCharSet) <> idx then
            S := S + FCharacterText[idx] + '.' + #13#10
          else
            S := S + '<!--Modified Text Here-->.' + #13#10;
        end;
        Result := 'CHARACTERS' + CRLF + S + CRLF;
        Exit;
      end;
    end
    else
      Result := 'CHARACTERS' + CRLF;
    Exit;
  end
  else // Whole Character set is under edit
    Result := 'CHARACTERS' + CRLF + ModTextTag + CRLF;
end;

function TCocoCharacters.GetCharacterSet(const SetTitle: string): string;
var
  idx: integer;
begin
  idx := FCharacterText.IndexOfName(SetTitle);
  if (idx > -1) then
    Result := FCharacterText.Values[SetTitle]
  else
    Result := 'Error';
end;


procedure TCocoCharacters.SetCharacterText(const Value: TStringList);
begin
  FCharacterText.Assign(Value);
end;

procedure TCocoCharacters.SetComments(const Value: string);
begin
  FComments := Value;
end;

procedure TCocoCharacters.SetEditingCharSet(const Value: string);
begin
  FEditingCharSet := Value;
end;

procedure TCocoCharacters.SetIsEditing(const Value: boolean);
begin
  FIsEditing := Value;
end;

end.
