{-----------------------------------------------------------------------------
 Unit Name: CocoImporter
 Author:    Administrator
 Purpose: Controls the import of the ATG for the purpose of GUI building.
-----------------------------------------------------------------------------}



unit CocoImporter;

interface

uses
  Windows, Messages, SysUtils, Classes, CompilerObj, pbCocoImportParser,
  pbStrLib;

type
  TCocoImporter = class(TObject)
  private
    { Private declarations }
    fpbCocoImportParser: TpbCocoImportParser;
    FCompiler: TCompiler;
    FSSLog: string;
    procedure SetCompiler(const Value: TCompiler);
    function GetSSLog: string;
  protected
    { Protected declarations }
    procedure ReportError(nr, line, col: Integer; pos: Integer);
  public
    { Public declarations }
    constructor Create;
    destructor Destroy; override;
    function ParseText(const S: string): boolean;
  published
    { Published declarations }
    property Compiler: TCompiler read FCompiler write SetCompiler;
    property Log: string read GetSSLog;
  end;

implementation

uses pbCocoImportScanner;

{ TCocoImporter }

constructor TCocoImporter.Create;
begin
  inherited;
end;

destructor TCocoImporter.Destroy;
begin
  inherited;
end;

{-----------------------------------------------------------------------------
  Procedure: TCocoImporter.ParseText
  Arguments: const S: String
  Result:    Boolean
  Description

  Returns false if there were errors.
-----------------------------------------------------------------------------}

function TCocoImporter.GetSSLog: string;
begin
  Result := FSSLog;
end;

function TCocoImporter.ParseText(const S: string): boolean;
var 
  oldname, NewName: string; // if the name of the compiler changes,
  // we need to refresh the GUI
begin
  Result := False;
  Assert(S <> ModTextTag, 'BUG:S=ModTextTag');
  if (S <> '') then
  begin
    Assert(FCompiler <> nil, #13#10 +
      'Compiler Object reference must be set in TCocoImporter.' + #13#10);
    oldName := FCompiler.CompilerName;
    FCompiler.ClearAll;
    fpbCocoImportParser := TpbCocoImportParser.Create;
    fpbCocoImportParser.errors := 0;
    fpbCocoImportParser.Error := ReportError;
    fpbCocoImportParser.SourceString := S;
    fpbCocoImportParser.Compiler := FCompiler;
    if FCompiler.OriginalATG = '' then
      FCompiler.OriginalATG := S;
    fpbCocoImportParser.Parse;
    Result := (fpbCocoImportParser.errors = 0);
    //  FSSLog := fpbCocoImportParser.lst.DataString;
    fpbCocoImportParser.Free;
    NewName := FCompiler.CompilerName;
    if (OldName <> NewName) then fCompiler.IsDirty := True
    else 
      fCompiler.IsDirty := False;
  end;
end;

procedure TCocoImporter.ReportError(nr, line, col, pos: Integer);
var 
  S: string;
begin
  case nr of
    {$I pbCocoImport.err}
  end; // Case
  S := S + 'Line ' + IntToStr(line) + CRLF + 'Column ' + IntToStr(col) + CRLF;
  FSSLog := FSSLog + #13#10 + S;
end;

procedure TCocoImporter.SetCompiler(const Value: TCompiler);
begin
  FCompiler := Value;
end;

end.
