{-----------------------------------------------------------------------------
 Unit Name: pbFileSystem
 Author:    Pat Connors

 Purpose:
  Base class for the file system - supposed to be reusable to act as the skeleton
  for a generic file system. So far, only used here.
-----------------------------------------------------------------------------}


unit pbFileSystem;

interface

uses
  Windows, Messages, SysUtils, Classes, Forms;

type
  tpbFileSystem = class(TComponent)
  private
    FFileName: string;
    FDefDir: string;
    FSaveDlgCaption: string;
    //    FDefFileExt: string;
    FOpenDlgCaption: string;
    FOnAfterMakeSnapShotCopy: TNotifyEvent;
    FOnAfterRenameOrReposition: TNotifyEvent;
    FOnBeforeRevertToMilestone: TNotifyEvent;
    FOnBeforeMakeSnapShotCopy: TNotifyEvent;
    FOnAfterRevertToMilestone: TNotifyEvent;
    FOnBeforeNew: TNotifyEvent;
    FOnBeforeMakeMilestone: TNotifyEvent;
    FOnBeforeAbandonChanges: TNotifyEvent;
    FOnBeforeClose: TNotifyEvent;
    FOnBeforeDocumentProperties: TNotifyEvent;
    FOnBeforeOpen: TNotifyEvent;
    FOnAfterMakeMilestone: TNotifyEvent;
    FOnAfterClose: TNotifyEvent;
    FOnAfterDocumentProperties: TNotifyEvent;
    FOnBeforeRenameOrReposition: TNotifyEvent;
    FOnAfterOpen: TNotifyEvent;
    FOnAfterAbandonChanges: TNotifyEvent;
    FOnAfterNew: TNotifyEvent;
    //    FFilter: string;
    FUseFilename: boolean;
    FFileHistory: TStringList;
    procedure SetDefDir(const Value: string);
    //    procedure SetDefFileExt(const Value: string);
    procedure SetOnAfterAbandonChanges(const Value: TNotifyEvent);
    procedure SetOnAfterClose(const Value: TNotifyEvent);
    procedure SetOnAfterDocumentProperties(const Value: TNotifyEvent);
    procedure SetOnAfterMakeMilestone(const Value: TNotifyEvent);
    procedure SetOnAfterMakeSnapShotCopy(const Value: TNotifyEvent);
    procedure SetOnAfterNew(const Value: TNotifyEvent);
    procedure SetOnAfterOpen(const Value: TNotifyEvent);
    procedure SetOnAfterRenameOrReposition(const Value: TNotifyEvent);
    procedure SetOnAfterRevertToMilestone(const Value: TNotifyEvent);
    procedure SetOnBeforeAbandonChanges(const Value: TNotifyEvent);
    procedure SetOnBeforeClose(const Value: TNotifyEvent);
    procedure SetOnBeforeDocumentProperties(const Value: TNotifyEvent);
    procedure SetOnBeforeMakeMilestone(const Value: TNotifyEvent);
    procedure SetOnBeforeMakeSnapShotCopy(const Value: TNotifyEvent);
    procedure SetOnBeforeNew(const Value: TNotifyEvent);
    procedure SetOnBeforeOpen(const Value: TNotifyEvent);
    procedure SetOnBeforeRenameOrReposition(const Value: TNotifyEvent);
    procedure SetOnBeforeRevertToMilestone(const Value: TNotifyEvent);
    procedure SetOpenDlgCaption(const Value: string);
    procedure SetSaveDlgCaption(const Value: string);
    //    procedure SetFilter(const Value: string);
    procedure SetUseFilename(const Value: boolean);
    procedure SetFileHistory(const Value: TStringList);
    { Private declarations }
  protected
    { Protected declarations }
    function Do_New: boolean; virtual; abstract;
    function Do_Open: boolean; virtual; abstract;
    function Do_Close: boolean; virtual; abstract;
    function Do_RenameOrReposition: boolean; virtual; abstract;
    function Do_MakeSnapShotCopy: boolean; virtual; abstract;
    function Do_MakeMilestone: boolean; virtual; abstract;
    function Do_RevertToMilestone: boolean; virtual; abstract;
    function Do_DocumentProperties: boolean; virtual; abstract;
    function Do_AbandonChanges: boolean; virtual; abstract;

    function CheckFileNameOK: boolean; virtual; abstract;

    procedure SaveFileHistory; virtual;
    procedure LoadFileHistory; virtual;
    function GetFileName: string; virtual;
    procedure SetFileName(const Value: string); virtual;

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure New;
    procedure Open;
    procedure Close;
    procedure RenameOrReposition;
    procedure MakeSnapShotCopy;
    procedure MakeMilestone;
    procedure RevertToMilestone;
    procedure DocumentProperties;
    procedure AbandonChanges;
    procedure AddFilenameToHistory;
  published
    { Published declarations }
    property FileName: string read GetFileName write SetFileName;
    property DefDir: string read FDefDir write SetDefDir;
    property OpenDlgCaption: string read FOpenDlgCaption write SetOpenDlgCaption;
    property SaveDlgCaption: string read FSaveDlgCaption write SetSaveDlgCaption;
    property FileHistory: TStringList read FFileHistory write SetFileHistory;
    property UseFilename: boolean read FUseFilename write SetUseFilename;
    property OnBeforeNew: TNotifyEvent read FOnBeforeNew write SetOnBeforeNew;
    property OnBeforeOpen: TNotifyEvent read FOnBeforeOpen write SetOnBeforeOpen;
    property OnBeforeClose: TNotifyEvent read FOnBeforeClose write SetOnBeforeClose;
    property OnBeforeRenameOrReposition: TNotifyEvent          
      read FOnBeforeRenameOrReposition write SetOnBeforeRenameOrReposition;
    property OnBeforeMakeSnapShotCopy: TNotifyEvent          
      read FOnBeforeMakeSnapShotCopy write SetOnBeforeMakeSnapShotCopy;
    property OnBeforeMakeMilestone: TNotifyEvent          
      read FOnBeforeMakeMilestone write SetOnBeforeMakeMilestone;
    property OnBeforeRevertToMilestone: TNotifyEvent          
      read FOnBeforeRevertToMilestone write SetOnBeforeRevertToMilestone;
    property OnBeforeDocumentProperties: TNotifyEvent          
      read FOnBeforeDocumentProperties write SetOnBeforeDocumentProperties;
    property OnBeforeAbandonChanges: TNotifyEvent    
      read FOnBeforeAbandonChanges write SetOnBeforeAbandonChanges;
    property OnAfterNew: TNotifyEvent read FOnAfterNew write SetOnAfterNew;
    property OnAfterOpen: TNotifyEvent read FOnAfterOpen write SetOnAfterOpen;
    property OnAfterClose: TNotifyEvent read FOnAfterClose write SetOnAfterClose;
    property OnAfterRenameOrReposition: TNotifyEvent    
      read FOnAfterRenameOrReposition write SetOnAfterRenameOrReposition;
    property OnAfterMakeSnapShotCopy: TNotifyEvent    
      read FOnAfterMakeSnapShotCopy write SetOnAfterMakeSnapShotCopy;
    property OnAfterMakeMilestone: TNotifyEvent    
      read FOnAfterMakeMilestone write SetOnAfterMakeMilestone;
    property OnAfterRevertToMilestone: TNotifyEvent    
      read FOnAfterRevertToMilestone write SetOnAfterRevertToMilestone;
    property OnAfterDocumentProperties: TNotifyEvent    
      read FOnAfterDocumentProperties write SetOnAfterDocumentProperties;
    property OnAfterAbandonChanges: TNotifyEvent    
      read FOnAfterAbandonChanges write SetOnAfterAbandonChanges;
  end;

implementation

{ tpbFileSystem }

//------------------------------------------------------------------------------

procedure tpbFileSystem.AbandonChanges;
begin
  if (Assigned(fOnBeforeAbandonChanges)) then
    fOnBeforeAbandonChanges(Self);
  if (Do_AbandonChanges) and (Assigned(fOnAfterAbandonChanges)) then
    fOnAfterAbandonChanges(Self);
end;

procedure tpbFileSystem.Close;
begin
  if (Assigned(fOnBeforeClose)) then
    fOnBeforeClose(Self);
  if (Do_Close) and (Assigned(fOnAfterClose)) then
    fOnAfterClose(Self);
end;

procedure tpbFileSystem.DocumentProperties;
begin
  if (Assigned(fOnBeforeDocumentProperties)) then
    fOnBeforeDocumentProperties(Self);
  if (Do_DocumentProperties) and (Assigned(fOnAfterDocumentProperties)) then
    fOnAfterDocumentProperties(Self);
end;

procedure tpbFileSystem.MakeMilestone;
begin
  if (Assigned(fOnBeforeMakeMilestone)) then
    fOnBeforeMakeMilestone(Self);
  if (Do_MakeMilestone) and (Assigned(fOnAfterMakeMilestone)) then
    fOnAfterMakeMilestone(Self);
end;

procedure tpbFileSystem.MakeSnapShotCopy;
begin
  if (Assigned(fOnBeforeMakeSnapShotCopy)) then
    fOnBeforeMakeSnapShotCopy(Self);
  if (Do_MakeSnapShotCopy) and (Assigned(fOnAfterMakeSnapShotCopy)) then
    fOnAfterMakeSnapShotCopy(Self);
end;

procedure tpbFileSystem.New;
begin
  if (Assigned(fOnBeforeNew)) then
    fOnBeforeNew(Self);
  if (Do_New) and (Assigned(fOnAfterNew)) then
    fOnAfterNew(Self);
end;

procedure tpbFileSystem.Open;
begin
  if (Assigned(fOnBeforeOpen)) then
    fOnBeforeOpen(Self);
  if (Do_Open) and (Assigned(fOnAfterOpen)) then
    fOnAfterOpen(Self);
end;

procedure tpbFileSystem.RenameOrReposition;
begin
  if (Assigned(fOnBeforeRenameOrReposition)) then
    fOnBeforeRenameOrReposition(Self);
  if (Do_RenameOrReposition) and (Assigned(fOnAfterRenameOrReposition)) then
    fOnAfterRenameOrReposition(Self);
end;

procedure tpbFileSystem.RevertToMilestone;
begin
  if (Assigned(fOnBeforeRevertToMilestone)) then
    fOnBeforeRevertToMilestone(Self);
  if (Do_RevertToMilestone) and (Assigned(fOnAfterRevertToMilestone)) then
    fOnAfterRevertToMilestone(Self);
end;

procedure tpbFileSystem.SetDefDir(const Value: string);
begin
  FDefDir := Value;
end;

procedure tpbFileSystem.SetFileName(const Value: string);
begin
  FFileName := Value;
end;

procedure tpbFileSystem.SetOnAfterAbandonChanges(const Value: TNotifyEvent);
begin
  FOnAfterAbandonChanges := Value;
end;

procedure tpbFileSystem.SetOnAfterClose(const Value: TNotifyEvent);
begin
  FOnAfterClose := Value;
end;

procedure tpbFileSystem.SetOnAfterDocumentProperties(const Value: TNotifyEvent);
begin
  FOnAfterDocumentProperties := Value;
end;

procedure tpbFileSystem.SetOnAfterMakeMilestone(const Value: TNotifyEvent);
begin
  FOnAfterMakeMilestone := Value;
end;

procedure tpbFileSystem.SetOnAfterMakeSnapShotCopy(const Value: TNotifyEvent);
begin
  FOnAfterMakeSnapShotCopy := Value;
end;

procedure tpbFileSystem.SetOnAfterNew(const Value: TNotifyEvent);
begin
  FOnAfterNew := Value;
end;

procedure tpbFileSystem.SetOnAfterOpen(const Value: TNotifyEvent);
begin
  FOnAfterOpen := Value;
end;

procedure tpbFileSystem.SetOnAfterRenameOrReposition(const Value: TNotifyEvent);
begin
  FOnAfterRenameOrReposition := Value;
end;

procedure tpbFileSystem.SetOnAfterRevertToMilestone(const Value: TNotifyEvent);
begin
  FOnAfterRevertToMilestone := Value;
end;

procedure tpbFileSystem.SetOnBeforeAbandonChanges(const Value: TNotifyEvent);
begin
  FOnBeforeAbandonChanges := Value;
end;

procedure tpbFileSystem.SetOnBeforeClose(const Value: TNotifyEvent);
begin
  FOnBeforeClose := Value;
end;

procedure tpbFileSystem.SetOnBeforeDocumentProperties(const Value: TNotifyEvent);
begin
  FOnBeforeDocumentProperties := Value;
end;

procedure tpbFileSystem.SetOnBeforeMakeMilestone(const Value: TNotifyEvent);
begin
  FOnBeforeMakeMilestone := Value;
end;

procedure tpbFileSystem.SetOnBeforeMakeSnapShotCopy(const Value: TNotifyEvent);
begin
  FOnBeforeMakeSnapShotCopy := Value;
end;

procedure tpbFileSystem.SetOnBeforeNew(const Value: TNotifyEvent);
begin
  FOnBeforeNew := Value;
end;

procedure tpbFileSystem.SetOnBeforeOpen(const Value: TNotifyEvent);
begin
  FOnBeforeOpen := Value;
end;

procedure tpbFileSystem.SetOnBeforeRenameOrReposition(const Value: TNotifyEvent);
begin
  FOnBeforeRenameOrReposition := Value;
end;

procedure tpbFileSystem.SetOnBeforeRevertToMilestone(const Value: TNotifyEvent);
begin
  FOnBeforeRevertToMilestone := Value;
end;

procedure tpbFileSystem.SetOpenDlgCaption(const Value: string);
begin
  FOpenDlgCaption := Value;
end;

procedure tpbFileSystem.SetSaveDlgCaption(const Value: string);
begin
  FSaveDlgCaption := Value;
end;


constructor tpbFileSystem.Create(AOwner: TComponent);
begin
  inherited;
  fFileHistory := TStringList.Create;
end;

procedure tpbFileSystem.SetUseFilename(const Value: boolean);
begin
  FUseFilename := Value;
end;

procedure tpbFileSystem.SetFileHistory(const Value: TStringList);
begin
  FFileHistory.Assign(Value);
end;

destructor tpbFileSystem.Destroy;
begin
  fFileHistory.Free;
  inherited;
end;

procedure tpbFileSystem.SaveFileHistory;
var
  S: string;
begin
  S := ChangeFileExt(Application.ExeName, '.hst');
  FFileHistory.SaveToFile(S);
end;

procedure tpbFileSystem.LoadFileHistory;
var
  S: string;
begin
  S := ChangeFileExt(Application.ExeName, '.hst');
  if (FileExists(S)) then
    FFileHistory.LoadFromFile(S);
end;

procedure tpbFileSystem.AddFilenameToHistory;
var
  idx: integer;
begin
  if (CheckFileNameOK) then
  begin
    idx := FFileHistory.IndexOf(FileName);
    if (idx > -1) then
    begin
      FFileHistory.Delete(idx);
    end;
    FFileHistory.Insert(0, FileName);
  end;
end;

function tpbFileSystem.GetFileName: string;
begin
  Result := FFileName;
end;

//procedure tpbFileSystem.SetDefFileExt(const Value: string);
//begin
//  FDefFileExt := Value;
//end;

//procedure tpbFileSystem.SetFilter(const Value: string);
//begin
//  FFilter := Value;
//end;



end.
