unit ParserUnitImportParserComponent;

interface

uses
  Windows, Messages, SysUtils, Classes, ParserUnitImportParser, pbStrLib,
  CompilerObj, FileUtil;

type
  TParserUnitImportParserComponent = class(TComponent)
  private
    { Private declarations }
    FParserUnitImportParser: TParserUnitImportParser;
    FInput: string;
    FOutput: string;
    FCompiler: TCompiler;
    procedure SetInput(const Value: string);
    procedure SetOutput(const Value: string);
    procedure SetCompiler(const Value: TCompiler);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(Aowner: TComponent); override;
    destructor Destroy; override;
    function Execute: boolean;
  published
    { Published declarations }
    property Input: string read FInput write SetInput;
    property Output: string read FOutput write SetOutput;
    property Compiler: TCompiler read FCompiler write SetCompiler;
  end;


function ExecuteParserUnitImportParser(aCompiler: TCompiler): boolean;


implementation

uses ParserUnitImportScanner;


function ExecuteParserUnitImportParser(aCompiler: TCompiler): boolean;
var 
  S: string;
begin
  S := RemoveBackSlash(aCompiler.TargetDirectory) + '\' +
    ExtractFileName(aCompiler.CompilerName) + 'Parser.pas';
  LoadString(S, S);
  with  TParserUnitImportParserComponent.Create(nil) do
    try
      Input := S;
      Compiler := aCompiler;
      Result := Execute;
    finally
      Free;
    end;
end;


{ TParserUnitImportParserComponent }

constructor TParserUnitImportParserComponent.Create(Aowner: TComponent);
begin
  inherited;
  FParserUnitImportParser := TParserUnitImportParser.Create;
end;

destructor TParserUnitImportParserComponent.Destroy;
begin
  FParserUnitImportParser.Free;
  inherited;
end;

function TParserUnitImportParserComponent.Execute: boolean;
var
  GramName: string;
begin
  Assert(FCompiler <> nil, 'BUG: NIL Compiler.');
  FParserUnitImportParser.SourceString := FInput;
  FParserUnitImportParser.Parse;
  FOutput := FParserUnitImportParser.lst.DataString;
  Result := FParserUnitImportParser.errors = 0;
  GramName := ' T' + FCompiler.CompilerName;
  if (Result) then
  begin
    FCompiler.DelphiSection.CreateSrc := FParserUnitImportParser.CreateSection;
    FCompiler.DelphiSection.DestroySrc := FParserUnitImportParser.DestroySection;
    FCompiler.DelphiSection.TypeDeclarations := FParserUnitImportParser.GlobalSection;
    FCompiler.DelphiSection.PrivateDecs := FParserUnitImportParser.PrivateSection;
    FCompiler.DelphiSection.ProtectedDecs := FParserUnitImportParser.ProtectedSection;
    FCompiler.DelphiSection.PublicDecs := FParserUnitImportParser.PublicSection;
    FCompiler.DelphiSection.PublishedDecs := FParserUnitImportParser.PublishedSection;
    FCompiler.DelphiSection.ArbitraryCodeSection :=
      ReplaceAll(FParserUnitImportParser.ArbitrarySourceSection, GramName, ' T-->Grammar');
    FCompiler.DelphiSection.ArbitraryCodeSection :=
      ReplaceAll(FCompiler.DelphiSection.ArbitraryCodeSection,
      FCompiler.CompilerName + '.err', '-->Grammar.err');
    //    S := 'Private'+#13#10+ FCompiler.DelphiSection.PrivateDecs;
    //    S := S +'Protected'+#13#10+ FCompiler.DelphiSection.ProtectedDecs;
    //    S := 'Public'+#13#10+ FCompiler.DelphiSection.PublicDecs;
    //    S := S +'Published'+#13#10+ FCompiler.DelphiSection.PublishedDecs;
    //    SaveString(S,'C:\temp\S.txt');
  end
  else
    raise Exception.Create(FOutput);
end;

procedure TParserUnitImportParserComponent.SetOutput(const Value: string);
begin
  FOutput := Value;
end;

procedure TParserUnitImportParserComponent.SetInput(const Value: string);
begin
  FInput := Value;
end;

procedure TParserUnitImportParserComponent.SetCompiler(const Value: TCompiler);
begin
  FCompiler := Value;
end;

end.
